

def formatted_title(vod) -> str:
    title = ""
    title += "[" + repr(vod["date"]) + "] "
    title += vod["tournament"] + " - "
    title += vod["players"][0]["alias"] + " "
    title += "(" + ", ".join(vod["players"][0]["characters"]) + ") "
    title += "vs "
    title += vod["players"][1]["alias"] + " "
    title += "(" + ", ".join(vod["players"][1]["characters"]) + ") "
    title += "- " + vod["round"] + " "
    title += "- Bo" + str(vod["best_of"])

    return title


def build_url(video_game, event: str = None, player1: str = None, player2: str = None, character1: str = None, character2: str = None, caster1: str = None, caster2: str = None) -> str:
    base_url = "https://vods.co/" + video_game
    if player1:
        base_url += "/player/" + player1
    if player2:
        base_url += "/player2/" + player2
    if event:
        base_url += "/event/" + event
    if character1:
        base_url += "/character/" + character1
    if character2:
        base_url += "/character2/" + character2
    if caster1:
        base_url += "/caster/" + character1
    if caster2:
        base_url += "/caster2/" + character2
    return base_url



import argparse
import json

from tqdm import tqdm

from vodscrepe import Scraper, formatted_title


def main():

    parser = argparse.ArgumentParser(
        description='Scrape the vods from http://vods.co/')
    parser.add_argument('game', type=str, default='melee',
                        help='The game whose vods will be scraped')
    parser.add_argument('-f', '--first', type=int, default=0,
                        help='First page of scraping')
    parser.add_argument('-l', '--last', type=int, default=1000,
                        help='Last page of scraping')
    parser.add_argument('-p1', '--player1', type=str, default="",
                        help='Player 1')
    parser.add_argument('-p2', '--player2', type=str, default="",
                        help='Player 2')
    parser.add_argument('-c1', '--character1', type=str, default="",
                        help='Character 1')
    parser.add_argument('-c2', '--character2', type=str, default="",
                        help='Character 2')
    parser.add_argument('-e', '--event', type=str, default="",
                        help='Event')
    parser.add_argument('-ca1', '--caster1', type=str, default="",
                        help='Caster 1')
    parser.add_argument('-ca2', '--caster2', type=str, default="",
                        help='Caster 2')
    parser.add_argument('-p', '--progress', type=bool, default=True,
                        help='Display scraping progress')
    parser.add_argument('-v', '--verbose', type=bool, default=False,
                        help='Display error statements')
    parser.add_argument('-j', '--json', type=bool, default=False,
                        help='Output vod with JSON format')
    parser.add_argument('-w', '--workers', type=int, default=10,
                        help='Number of workers')
    parser.add_argument('-pw', '--page-workers', type=int, default=1,
                        help='Number of page workers')

    args = parser.parse_args()

    s = Scraper(args.game, event=args.event, player1=args.player1, player2=args.player2, character1=args.character1,
                character2=args.character2, caster1=args.caster1, caster2=args.caster2, num_workers=args.workers, num_page_workers=args.page_workers)

    pages = range(args.first, min(s.num_pages - 1, args.last))
    try:
        for vod in s.scrape(pages, show_progress=args.progress, verbose=args.verbose):
            if vod is not None:
                if args.json:
                    tqdm.write(json.dumps(vod, indent=None))
                else:
                    tqdm.write(formatted_title(vod))
    except KeyboardInterrupt:
        tqdm.write("Scraping terminated.")


if __name__ == '__main__':
    main()






class InvalidVideoException(Exception):
    def __init__(self, *args):
        message = "Invalid Video: vods.co vod '%s' links to invalid video" % args
        super(InvalidVideoException, self).__init__(message)


class InvalidPageException(Exception):
    def __init__(self, *args):
        message = "Invalid Page: vods.co game '%s' does not have page '%i'" % args
        super(InvalidPageException, self).__init__(message)



import re
import sys
from concurrent.futures import as_completed
from queue import Queue

from bs4 import BeautifulSoup, SoupStrainer
from requests_futures.sessions import FuturesSession
from tqdm import tqdm

from .aliases import guess_character
from .errors import InvalidVideoException
from .utils import build_url

stderr = lambda *args: print(file=sys.stderr, *args)


class Scraper:
    def __init__(self, video_game, event='', player1='', player2='', character1='', character2='', caster1='', caster2='',  num_workers=10, num_page_workers=2):
        self.video_game = video_game
        self.event = event
        self.player1 = player1
        self.player2 = player2
        self.character1 = character1
        self.character2 = character2
        self.caster1 = caster1
        self.caster2 = caster2

        self.base_url = build_url(
            video_game, event, player1, player2, character1, character2, caster1, caster2)

        self.num_workers = num_workers
        self.num_page_workers = min(num_page_workers, self.num_workers)
        self.session = FuturesSession(max_workers=self.num_workers)

        page_content = self.request(self.base_url).result().content
        page_soup = BeautifulSoup(page_content, "lxml")
        sections = [h2.parent for h2 in page_soup.findChildren(
            "h2", class_="pane-title block-title")]

        self.characters = [o.getText()[1:]
                           for o in sections[0].findChildren("option")[1:]]
        self.players = [o.getText()[1:]
                        for o in sections[1].findChildren("option")[1:]]
        self.events = [o.getText()[1:]
                       for o in sections[2].findChildren("option")[1:]]
        self.casters = [o.getText()[1:]
                        for o in sections[3].findChildren("option")[1:]]

        self.num_pages = 1
        last_page_tag = page_soup.findChild("a", title="Go to last page")
        if last_page_tag is not None:
            self.num_pages = int(
                re.search(r"page=([\d]+)", last_page_tag["href"]).group(1))

    def request(self, url):
        headers = {'Accept-Encoding': 'gzip'}

        response = self.session.get(url, headers=headers)
        return response

    def scrape_vod_page(self, vod_id, vod_request, verbose: bool):
        vod_content = vod_request.result().content
        vod_strainer = SoupStrainer('div', class_="region-inner clearfix")
        vod_soup = BeautifulSoup(
            vod_content, "lxml", parse_only=vod_strainer)
        content = vod_soup.findChild(recursive=False)

        try:
            video_ids = [re.search(r"^([^?]*)", v["data-vod"]).group(1) for v in content.findChildren(
                "div", class_="js-video widescreen", recursive=False)]
            if len(video_ids) == 0:
                raise InvalidVideoException(vod_id)

            casters_tag = content.findChild("div", class_="field-items")
            casters = [{"alias": c.getText()} for c in casters_tag.findChildren(
                recursive=False)] if casters_tag is not None else []
            return (video_ids, casters)
        except KeyError:
            raise InvalidVideoException(vod_id)

    def scrape_page(self, page_request, verbose: bool):
        page_content = page_request.result().content
        page_strainer = SoupStrainer("table")
        page_soup = BeautifulSoup(
            page_content, "lxml", parse_only=page_strainer)

        vod_requests = [self.request(tr.findChild("a")["href"])
                        for tr in page_soup.findChildren("tr")]

        for table in page_soup.findChildren(recursive=False):
            date = table.caption.span.getText()
            for i, row in enumerate(table.tbody.findChildren(recursive=False)):
                cells = row.findChildren(recursive=False)

                try:
                    vod_id = re.search(
                        r".*\/(.*)", cells[1].a["href"]).group(1)

                    try:
                        best_of = int(re.search(
                            r"Bo([\d]*)", cells[3].getText()).group(1))
                    except AttributeError:
                        continue

                    players = []
                    player = {"alias": "Unknown", "characters": []}
                    for tag in cells[1].a.span.findChildren(recursive=False):
                        if tag.name == u'b':
                            if len(player["characters"]) != 0:
                                players.append(player)
                                player = {"alias": "Unknown",
                                          "characters": []}
                            player["alias"] = tag.getText()
                        elif tag.name == u'img':
                            player["characters"].append(
                                guess_character(tag["src"][24:-4]))
                    players.append(player)

                    video_ids, casters = self.scrape_vod_page(
                        vod_id, vod_requests[i], verbose)

                    yield {
                        "vod_id": vod_id,
                        "video_ids": video_ids,
                        "date": date,
                        "tournament": re.search(r"[^\s].*[^\s]", cells[0].getText()).group(),
                        "players": players,
                        "casters": casters,
                        "round": re.search(r"[^\s].*[^\s]", cells[4].getText()).group(),
                        "best_of": best_of
                    }
                except InvalidVideoException as e:
                    if verbose:
                        stderr(e)

    def scrape(self, pages=None, show_progress=False, verbose=False):
        if pages is None:
            pages = range(self.num_pages - 1)

        self.num_page_workers = min(self.num_page_workers, len(pages))

        request_queue = Queue(self.num_page_workers)
        for i in range(self.num_page_workers):
            request_queue.put(self.request(
                self.base_url + "?page=" + str(pages[i])))

        iter_pages = pages
        if show_progress:
            iter_pages = tqdm(iter_pages, position=1,
                              unit='pages', desc="All vods")

        for page in iter_pages:

            vods = self.scrape_page(request_queue.get(), verbose)
            if show_progress:
                vods = tqdm(vods, position=0, unit='vods',
                            desc="Page %d" % page, total=60)

            for vod in vods:
                yield vod

            request_queue.put(self.request(
                self.base_url + "?page=" + str(page + self.num_page_workers)))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vodscrepe']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.8.2,<5.0.0',
 'lxml>=4.4.2,<5.0.0',
 'requests_futures>=1.0.0,<2.0.0',
 'tqdm>=4.41.1,<5.0.0']

entry_points = \
{'console_scripts': ['vodscrepe = vodscrepe.console:main']}

setup_kwargs = {
    'name': 'vodscrepe',
    'version': '1.2.5',
    'description': 'https://vods.co/ vod scraper',
    'long_description': '# `vodscrepe`\n\n[![](https://img.shields.io/pypi/v/vodscrepe.svg?style=flat)](https://pypi.org/pypi/vodscrepe/)\n[![](https://img.shields.io/pypi/dw/vodscrepe.svg?style=flat)](https://pypi.org/pypi/vodscrepe/)\n[![](https://img.shields.io/pypi/pyversions/vodscrepe.svg?style=flat)](https://pypi.org/pypi/vodscrepe/)\n[![](https://img.shields.io/pypi/format/vodscrepe.svg?style=flat)](https://pypi.org/pypi/vodscrepe/)\n[![](https://img.shields.io/pypi/l/vodscrepe.svg?style=flat)](https://github.com/dawsonbooth/vodscrepe/blob/master/LICENSE)\n\n# Description\n\nThis PyPI package is best described as a tool for scraping the [vods.co](https://vods.co/) website. Currently, the package only supports Super Smash Bros. Melee vods.\n\n# Installation\n\nWith [Python](https://www.python.org/downloads/) installed, simply run the following command to add the package to your project.\n\n```bash\npip install vodscrepe\n```\n\n# Usage\n\nThe following is an example usage of the package, which is also included in the repo as `example.py`:\n\n```python\nfrom tqdm import tqdm\n\nfrom vodscrepe import Scraper, formatted_title\n\ns = Scraper(\'melee\')\n\ntry:\n    for vod in s.scrape(show_progress=True):\n        tqdm.write(formatted_title(vod))\nexcept KeyboardInterrupt:\n    tqdm.write("Scraping terminated.")\n```\n\nThis example lists information about the vods from the most recent to the last page in the following fashion:\n\n```bash\npython example.py > sets.txt\n```\n\nThen, the `sets.txt` file becomes populated with vod information...\n\n```txt\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs Hungrybox (Jigglypuff) - Grand Finals - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs n0ne (Captain Falcon) - Losers Finals - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs Gahtzu (Captain Falcon) - Losers Semis - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Hungrybox (Jigglypuff) vs n0ne (Captain Falcon) - Winners Finals - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Panda (FL) (Fox) vs Gahtzu (Captain Falcon) - Losers Quarters - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs Chef Rach (Captain Falcon) - Losers Quarters - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - n0ne (Captain Falcon) vs Panda (FL) (Fox) - Winners Semis - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs Hungrybox (Jigglypuff) - Winners Semis - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Krudo (Sheik) vs Gahtzu (Captain Falcon) - Losers Top 8 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - blankshooter744 (Fox) vs Chef Rach (Captain Falcon) - Losers Round 5 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Leighton (Jigglypuff) vs Prof (Marth) - Losers Round 5 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Sinbad (Sheik) vs Krudo (Sheik) - Losers Round 5 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Wevans (Samus) vs Gahtzu (Captain Falcon) - Losers Round 5 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Sinbad (Sheik) vs Dom (FL) (Marth) - Losers Round 4 - Bo5\n[\'2020-03-15\'] CEO Dreamland 2020 - Colbol (Fox) vs Gahtzu (Captain Falcon) - Winners Quarters - Bo5\nScraping terminated.\n```\n\n...while the terminal details the progress:\n\n```bash\nAll vods:   0%|                                              | 0/331 [00:07<?, ?pages/s]\nPage 0:  25%|██████████████                                  | 15/60 [00:07<00:12,  3.07vods/s]\n```\n\n# License\n\nThis software is released under the terms of [MIT license](LICENSE).\n',
    'author': 'Dawson Booth',
    'author_email': 'pypi@dawsonbooth.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dawsonbooth/vodscrepe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
